/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "cviewportwatcher.h"

#include <QScrollArea>
#include <QScrollBar>

#include <QDebug>


CViewPortWatcher::CViewPortWatcher(QScrollArea *scrollArea, QObject * parent):QObject(parent),
    ScrollArea(scrollArea),
    SceneRect(),
    Scale(1)
{
    connect(ScrollArea->verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(viewPortChanged()));
    connect(ScrollArea->horizontalScrollBar(), SIGNAL(valueChanged(int)), SLOT(viewPortChanged()));
}



void CViewPortWatcher::setSceneRect(const QRectF &rect)
{
    SceneRect = rect;
    viewPortChanged();
}



void CViewPortWatcher::setScale(qreal scl)
{
    Scale = scl;
    viewPortChanged();
}



void CViewPortWatcher::viewPortPositionChanged(const QPointF &pos)
{
    //Устанавливаем скролбары так, чтобы точка pos была по середине экрана
    //pos содержит относительные координаты, поэтому перед использованием их нужно преобразовать

    qreal x = SceneRect.width() * pos.x() * Scale - ScrollArea->viewport()->width() / 2.0;
    qreal y = SceneRect.height() * pos.y() * Scale - ScrollArea->viewport()->height() / 2.0;

    ScrollArea->horizontalScrollBar()->setValue(x);
    ScrollArea->verticalScrollBar()->setValue(y);

    viewPortChanged();
}



void CViewPortWatcher::viewPortChanged()
{
    qreal w  = (ScrollArea->viewport()->rect().width() < SceneRect.width() * Scale) ?
          qreal(ScrollArea->viewport()->rect().width()) / Scale :
          SceneRect.width();

    qreal h  = (ScrollArea->viewport()->rect().height() < SceneRect.height() * Scale) ?
          qreal(ScrollArea->viewport()->rect().height()) / Scale :
          SceneRect.height();

    qreal x = qreal(ScrollArea->horizontalScrollBar()->value()) / Scale;

    qreal y = qreal(ScrollArea->verticalScrollBar()->value()) / Scale;

    //qDebug() << "visible Rect" << QRectF(x, y, w, h);

    const int dl = 2;//Fine tune position

    emit visibleRectChanged(QRectF(x, y, w-dl, h-dl));
}
